const fs = require('fs');
const path = require('path');
const puppeteer = require('puppeteer');

// Create screenshots directory if it doesn't exist
const screenshotsDir = path.join(__dirname, '../screenshots');
if (!fs.existsSync(screenshotsDir)) {
  fs.mkdirSync(screenshotsDir, { recursive: true });
}

// Helper function for waiting
const wait = ms => new Promise(resolve => setTimeout(resolve, ms));

async function captureScreenshots() {
  // Launch browser
  const browser = await puppeteer.launch({
    headless: "new",
    args: ['--no-sandbox', '--disable-setuid-sandbox']
  });
  
  // Create new page
  const page = await browser.newPage();
  
  // Set viewport
  await page.setViewport({ width: 1280, height: 800 });
  
  // Create test HTML file
  const testHtml = `
    <!DOCTYPE html>
    <html>
    <head>
      <title>JSON:API Explorer Test</title>
      <meta charset="utf-8">
      <meta name="viewport" content="width=device-width, initial-scale=1">
      <script>
        // Mock the Drupal settings for testing purposes
        window.drupalSettings = {
          jsonapi_explorer: {
            base_url: 'https://example.com/jsonapi/'
          }
        };
      </script>
    </head>
    <body>
      <div id="jsonapi-explorer-root" data-base-url="https://example.com/jsonapi/">
        <div class="loading">
          <div class="loading-spinner"></div>
          <h2>Loading JSON:API Explorer</h2>
          <p>The interface is initializing.</p>
        </div>
      </div>
      <script src="../build/static/js/main.js"></script>
    </body>
    </html>
  `;
  
  fs.writeFileSync(path.join(__dirname, 'test.html'), testHtml);

  try {
    console.log('Taking screenshots...');
    
    // Capture Main Query Builder Tab
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    // Take a screenshot of the query builder tab
    console.log('Capturing query builder tab...');
    await page.screenshot({
      path: path.join(screenshotsDir, 'query-builder-tab.png'),
      fullPage: true,
    });
    
    // Capture Authentication Section
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    // Find and open the Authentication section
    console.log('Capturing authentication section...');
    
    // Use evaluation to find the auth details element and click it
    const authDetailsOpen = await page.evaluate(() => {
      const authDetails = document.querySelector('.authentication-details');
      if (authDetails) {
        authDetails.open = true;
        return true;
      }
      return false;
    });
    
    if (authDetailsOpen) {
      // Wait for animation
      await wait(500);
      
      // Take a screenshot of the authentication section
      await page.screenshot({
        path: path.join(screenshotsDir, 'auth-section.png'),
        fullPage: true,
      });
    } else {
      console.log('Could not find auth details section');
    }
    
    // Capture Code Examples Tab
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    console.log('Capturing code examples tab...');
    // Click on the code examples tab
    await page.evaluate(() => {
      const tabs = Array.from(document.querySelectorAll('.query-panel-tab'));
      const codeTab = tabs.find(tab => tab.textContent.includes('</>'));
      if (codeTab) codeTab.click();
    });
    
    // Wait for animation
    await wait(500);
    
    // Take a screenshot of the code examples tab
    await page.screenshot({
      path: path.join(screenshotsDir, 'code-examples-tab.png'),
      fullPage: true,
    });
    
    console.log('Screenshots captured successfully');
  } catch (error) {
    console.error('Error taking screenshots:', error);
  } finally {
    // Clean up
    fs.unlinkSync(path.join(__dirname, 'test.html'));
    await browser.close();
  }
}

// Run the function
captureScreenshots().catch(console.error);