const fs = require('fs');
const path = require('path');

// Create screenshots directory if it doesn't exist
const screenshotsDir = path.join(__dirname, 'screenshots');
if (!fs.existsSync(screenshotsDir)) {
  fs.mkdirSync(screenshotsDir, { recursive: true });
}

// Helper function for waiting
const wait = ms => new Promise(resolve => setTimeout(resolve, ms));

describe('JSON:API Explorer Visual Test', () => {
  // Start local server for testing
  let server;
  
  beforeAll(async () => {
    // Create simple HTML file to load the built app
    const testHtml = `
      <!DOCTYPE html>
      <html>
      <head>
        <title>JSON:API Explorer Test</title>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <script>
          // Mock the Drupal settings for testing purposes
          window.drupalSettings = {
            jsonapi_explorer: {
              base_url: 'https://example.com/jsonapi/'
            }
          };
        </script>
      </head>
      <body>
        <div id="jsonapi-explorer-root" data-base-url="https://example.com/jsonapi/">
          <div class="loading">
            <div class="loading-spinner"></div>
            <h2>Loading JSON:API Explorer</h2>
            <p>The interface is initializing.</p>
          </div>
        </div>
        <script src="../build/static/js/main.js"></script>
      </body>
      </html>
    `;
    
    fs.writeFileSync(path.join(__dirname, 'test.html'), testHtml);
    
    // Set viewport to a desktop size
    await page.setViewport({ width: 1280, height: 800 });
  });
  
  afterAll(async () => {
    // Clean up test file
    fs.unlinkSync(path.join(__dirname, 'test.html'));
  });

  // Test the interface in light mode
  it('should capture the app layout', async () => {
    // Load the test HTML file
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    // Take a screenshot of the full interface
    await page.screenshot({
      path: path.join(screenshotsDir, 'jsonapi-explorer-layout.png'),
      fullPage: true,
    });
    
    // Check if URL display container exists and is visible
    const urlDisplay = await page.$('.url-display-container');
    expect(urlDisplay).not.toBeNull();
    
    // Verify URL display is not in a tab content that can be hidden
    const urlDisplayInTabContent = await page.evaluate(() => {
      const urlDisplay = document.querySelector('.url-display-container');
      if (!urlDisplay) return true;
      
      // Check if the URL display is inside a tab content div
      const tabContent = urlDisplay.closest('.tab-content-section');
      return !!tabContent;
    });
    
    expect(urlDisplayInTabContent).toBe(false);
  });
  
  // Test tab navigation
  it('should verify tabs and persistent URL display', async () => {
    // Load the test HTML file
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    // Check if URL tab is not present
    const urlTabContent = await page.evaluate(() => {
      const tabs = Array.from(document.querySelectorAll('.query-panel-tab'));
      return tabs.some(tab => tab.textContent.trim() === 'URL');
    });
    expect(urlTabContent).toBe(false);
    
    // Get all visible tabs
    const tabLabels = await page.evaluate(() => {
      const tabs = Array.from(document.querySelectorAll('.query-panel-tab'));
      return tabs.map(tab => tab.textContent.trim());
    });
    
    // Verify correct tabs are present (URL should not be among them)
    expect(tabLabels).toContain('Fields');
    expect(tabLabels).toContain('Filters');
    expect(tabLabels).toContain('Includes');
    expect(tabLabels).toContain('Sort');
    expect(tabLabels).toContain('</>');
    expect(tabLabels).not.toContain('URL');
    
    // Take a screenshot of the tabs
    await page.screenshot({
      path: path.join(screenshotsDir, 'jsonapi-explorer-tabs.png'),
      clip: {
        x: 0,
        y: 0,
        width: 1280,
        height: 200
      }
    });
  });

  // Test responsive layout
  it('should capture responsive layout', async () => {
    // Set viewport to a mobile size
    await page.setViewport({ width: 425, height: 800 });
    
    // Load the test HTML file
    await page.goto(`file://${path.join(__dirname, 'test.html')}`, {
      waitUntil: 'networkidle0',
    });
    
    // Wait for the app to initialize
    await wait(5000);
    
    // Take a screenshot of the responsive layout
    await page.screenshot({
      path: path.join(screenshotsDir, 'jsonapi-explorer-responsive.png'),
      fullPage: true,
    });
  });
});