# JSON:API Query Builder Development Guide

This document provides information for developers working on the JSON:API Query Builder module.

## Development Setup

### Requirements

- PHP 8.0 or higher
- Composer
- Node.js 16 or higher
- Yarn package manager

### Local Development Environment

1. Clone the repository:
   ```bash
   git clone [repository-url] jsonapi_query_builder
   cd jsonapi_query_builder
   ```

2. Install PHP dependencies:
   ```bash
   composer install
   ```

3. Install JavaScript dependencies:
   ```bash
   cd js/react-app
   yarn install
   ```

4. Build the React application:
   ```bash
   yarn build
   ```

5. Start the development server:
   ```bash
   yarn start
   ```

## Testing

The module includes both PHP and JavaScript tests. You can run them using Composer scripts from the module's root directory or directly using the underlying tools.

### Quick Reference (Composer Scripts)

Run these commands from the module's root directory:

```bash
# PHP Tests
composer test                  # Run all PHP tests (CS, PHPStan, Unit, Functional)
composer phpcs                 # Run code style checks
composer phpunit               # Run unit tests
composer phpunit-functional    # Run functional tests

# JavaScript Tests
composer test-js               # Run basic JS tests (screenshots)
composer test-js-screenshots   # Run headless screenshot tests
composer test-js-screenshots-interactive  # Run interactive screenshot tests (shows browser)
composer test-js-e2e-full      # Run full E2E tests with temporary Drupal install

# Run all tests (both PHP and JS)
composer test-all
```

### JavaScript Tests

The module includes different types of JavaScript tests for the React application:

#### Screenshot Tests

These tests capture screenshots of the UI for documentation without asserting functionality.

```bash
# Using Composer (from module root):
composer test-js-screenshots
composer test-js-screenshots-interactive  # Shows browser

# Or directly:
cd js/react-app && yarn test:screenshot
cd js/react-app && yarn test:screenshot:interactive
```

#### End-to-End Tests

E2E tests require a Drupal environment with the module installed. We provide multiple ways to run these tests:

1. **Full E2E Test with Temporary Drupal** (Recommended for CI/CD):
   ```bash
   # Using Composer (from module root):
   composer test-js-e2e-full
   
   # Or directly:
   cd js/react-app && yarn test:e2e:full
   ```
   This script:
   - Creates a temporary Drupal installation
   - Installs the module and dependencies
   - Runs tests against this environment
   - Captures screenshots
   - Cleans up when done

2. **Against Your Own Drupal Site**:
   ```bash
   # Using Composer (from module root):
   TEST_URL=http://your-drupal-site.test TEST_USERNAME=admin TEST_PASSWORD=yourpassword composer test-js-e2e
   TEST_URL=http://your-drupal-site.test TEST_USERNAME=admin TEST_PASSWORD=yourpassword composer test-js-e2e-interactive
   
   # Or directly:
   cd js/react-app
   export TEST_URL=http://your-drupal-site.test
   export TEST_USERNAME=your_admin_username
   export TEST_PASSWORD=your_admin_password
   
   # Run in headless mode
   yarn test:e2e
   
   # Or run in interactive mode (shows browser)
   yarn test:e2e:interactive
   ```

### PHP Tests

The module includes various PHP tests for the Drupal integration:

```bash
# Using Composer (from module root):
composer phpcs                 # Check coding standards
composer phpstan               # Static analysis
composer phpunit               # Unit tests
composer phpunit-functional    # Functional tests
composer phpunit-javascript    # JavaScript-integrated tests
composer test                  # Run all standard PHP tests

# Or directly (from Drupal root):
vendor/bin/phpunit -c core modules/contrib/jsonapi_query_builder
```

## Development Best Practices

### JavaScript Coding Standards

- Follow ESLint rules
- Use functional components with React hooks
- Add JSDoc comments for functions and components

### PHP Coding Standards

- Follow Drupal coding standards
- Pass PHP_CodeSniffer checks
- Add PHPDoc comments for methods and classes

### Commit Guidelines

- Write clear commit messages describing the change
- Include "why" in commit messages, not just "what"
- Reference issue numbers when appropriate

### Pull Request Process

- Create a feature branch from main
- Run all tests before submitting
- Address all code review feedback
- Ensure documentation is updated

## Documentation

- Update documentation when adding or changing features
- Document both user-facing features and developer APIs
- Add screenshots for UI components

## Project Structure

- `js/react-app/`: React application for the query builder interface
- `src/Controller/`: Drupal controllers for the module
- `templates/`: Twig templates for rendering
- `tests/`: Test files (both PHP and JavaScript)

## Getting Help

- Report issues on the project repository
- Reach out in the Drupal Slack #json-api channel
