import React from 'react';
import { createRoot } from 'react-dom/client';
import './index.css';
import './gin.css'; // Import Gin CSS after index.css for overrides
import App from './App';

// Wrap initialization in a function to make debugging easier
function initializeApp() {
  try {
    console.log('Initializing JSON:API Query Builder application...');
    
    // Log document state for debugging
    console.log('Document ready state:', document.readyState);
    console.log('Document body exists:', !!document.body);
    
    // Get the base URL from the data attributes
    // Try both the new ID and legacy ID to handle both
    let container = document.getElementById('jsonapi-query-builder-root');
    
    // For debugging, log all possible root elements
    const allPossibleContainers = [
      document.getElementById('jsonapi-query-builder-root'),
      document.getElementById('jsonapi-explorer-root')
    ];
    console.log('Possible containers:', allPossibleContainers.map(el => el ? el.id : 'null'));
    
    // Add safeguards against missing elements
    if (!container) {
      console.error('Error: Could not find the container element with ID "jsonapi-query-builder-root"');
      
      // Check if we need to fall back to the old ID
      const legacyContainer = document.getElementById('jsonapi-explorer-root');
      if (legacyContainer) {
        console.log('Found legacy container with ID "jsonapi-explorer-root", using it instead');
        container = legacyContainer;
      } else {
        // If neither container exists, show an error
        console.error('Could not find any valid container element for the app to mount on');
        
        // Try to show error on page
        if (document.body) {
          const errorDiv = document.createElement('div');
          errorDiv.style.color = 'red';
          errorDiv.style.padding = '20px';
          errorDiv.style.fontWeight = 'bold';
          errorDiv.textContent = 'Error: Could not initialize the JSON:API Query Builder application.';
          document.body.appendChild(errorDiv);
        } else {
          console.error('Cannot append error element - document.body is not available');
        }
        return; // Exit initialization
      }
    }
    
    // Safely get the base URL
    let baseUrl = '';
    try {
      baseUrl = container.getAttribute('data-base-url') || '';
      console.log('Got base URL from container:', baseUrl);
    } catch (e) {
      console.error('Error getting data-base-url attribute:', e);
      baseUrl = '/jsonapi/'; // Default fallback
    }
    
    // If there's no trailing slash, add one
    if (baseUrl && !baseUrl.endsWith('/')) {
      baseUrl += '/';
    }
    
    // Log for debugging
    console.log('Initializing JSON:API Query Builder with base URL:', baseUrl);
    
    // Create root and render app
    try {
      const root = createRoot(container);
      root.render(
        <React.StrictMode>
          <App baseUrl={baseUrl} />
        </React.StrictMode>
      );
      console.log('React app successfully rendered');
    } catch (e) {
      console.error('Error rendering React app:', e);
    }
  } catch (e) {
    console.error('Uncaught error during initialization:', e);
  }
}

// Call the initialization function
initializeApp();