#!/bin/bash

# Run tests for JSON:API Query Builder module

MODULE_DIR=$(pwd)
echo "Testing module in: $MODULE_DIR"

# Check if composer is installed
if ! command -v composer &> /dev/null; then
  echo "Error: Composer is not installed. Please install Composer and try again."
  exit 1
fi

# Verify module structure
echo "Verifying module structure..."

# Check essential files
REQUIRED_FILES=(
  "jsonapi_query_builder.info.yml"
  "jsonapi_query_builder.routing.yml"
  "jsonapi_query_builder.module"
  "jsonapi_query_builder.permissions.yml"
  "src/Controller/ExplorerController.php"
  "templates/jsonapi-query-builder.html.twig"
)

MISSING_FILES=0
for file in "${REQUIRED_FILES[@]}"; do
  if [ ! -f "$MODULE_DIR/$file" ]; then
    echo "Missing required file: $file"
    MISSING_FILES=1
  else
    echo "✓ Found $file"
  fi
done

if [ $MISSING_FILES -eq 1 ]; then
  echo "Error: Some required module files are missing."
  exit 1
fi

# Verify React app is built
if [ ! -d "$MODULE_DIR/js/react-app/build" ]; then
  echo "Building React application..."
  cd "$MODULE_DIR/js" && ./build.sh
  if [ $? -ne 0 ]; then
    echo "Error: Failed to build React application."
    exit 1
  fi
else
  echo "✓ React application is already built"
fi

# Run code quality checks
echo "Running code quality checks..."

# Ensure dependencies are installed
if [ ! -d "$MODULE_DIR/vendor" ]; then
  echo "Installing dependencies..."
  cd "$MODULE_DIR" && composer install
  if [ $? -ne 0 ]; then
    echo "Error: Failed to install dependencies."
    exit 1
  fi
fi

# Run PHP code sniffer
echo "Running PHP CodeSniffer..."
cd "$MODULE_DIR" && composer phpcs
PHPCS_RESULT=$?

# Run PHPStan
echo "Running PHPStan..."
cd "$MODULE_DIR" && composer phpstan
PHPSTAN_RESULT=$?

# Report results
echo "Test Results:"
echo "=============="

if [ $PHPCS_RESULT -eq 0 ]; then
  echo "✓ PHP CodeSniffer: PASSED"
else
  echo "✗ PHP CodeSniffer: FAILED"
fi

if [ $PHPSTAN_RESULT -eq 0 ]; then
  echo "✓ PHPStan: PASSED"
else
  echo "✗ PHPStan: FAILED"
fi

if [ $PHPCS_RESULT -eq 0 ] && [ $PHPSTAN_RESULT -eq 0 ]; then
  echo "All tests PASSED!"
  exit 0
else
  echo "Some tests FAILED!"
  exit 1
fi