#!/bin/bash

# CI Test script for the JSON:API Query Builder module
# This script will install Drupal and the module in a headless environment for CI testing

set -e  # Exit immediately if a command exits with a non-zero status
set -x  # Print commands and their arguments as they are executed

# Set variables
MODULE_NAME="jsonapi_query_builder"
TEST_DIR="/tmp/drupal_test_$MODULE_NAME"
MODULE_DIR=$(pwd)

# Create test directory
echo "Creating test directory at $TEST_DIR..."
mkdir -p "$TEST_DIR"
cd "$TEST_DIR"

# Create a new Drupal project
echo "Creating new Drupal project..."
composer create-project drupal/recommended-project:^10.1 . --no-interaction --no-progress

# Install required dependencies
echo "Installing required modules..."
composer require drupal/jsonapi_schema:^1.0@dev --no-interaction --no-progress

# Install Drush for site management
echo "Installing Drush..."
composer require drush/drush --no-interaction --no-progress

# Create module directory
echo "Setting up module directory..."
MODULE_TARGET_DIR="web/modules/contrib/$MODULE_NAME"
mkdir -p "$MODULE_TARGET_DIR"

# Copy module files
echo "Copying module files..."
cp -r "$MODULE_DIR"/* "$MODULE_TARGET_DIR/"
# Copy hidden files too
find "$MODULE_DIR" -maxdepth 1 -name ".*" -type f -exec cp {} "$MODULE_TARGET_DIR/" \; 2>/dev/null || true

# Remove unnecessary files from module directory for testing
echo "Cleaning up module directory..."
rm -rf "$MODULE_TARGET_DIR/vendor" "$MODULE_TARGET_DIR/release" "$MODULE_TARGET_DIR/node_modules"

# Install Drupal in headless mode
echo "Installing Drupal..."
vendor/bin/drush site:install --yes --db-url=sqlite://sites/default/files/.ht.sqlite --account-name=admin --account-pass=admin --site-name="JSON:API Query Builder Test" minimal

# Enable required modules
echo "Enabling required modules..."
vendor/bin/drush en -y jsonapi jsonapi_schema

# Enable our module
echo "Enabling $MODULE_NAME module..."
vendor/bin/drush en -y $MODULE_NAME

# Check if module routes exist
echo "Verifying module routes..."
ROUTE_CHECK=$(vendor/bin/drush route:info $MODULE_NAME.app 2>/dev/null)
if [[ $ROUTE_CHECK == *"$MODULE_NAME.app"* ]]; then
  echo "SUCCESS: Route '$MODULE_NAME.app' found."
else
  echo "ERROR: Route '$MODULE_NAME.app' not found."
  exit 1
fi

# Create some test content
echo "Creating test content..."
vendor/bin/drush entity:create node --bundle=article --values='{"title":"Test Article 1"}'

# Run basic functional test - accessing the module URL
echo "Testing module URL access..."
# Use curl to access the module route
vendor/bin/drush user:login admin --no-browser | grep -o 'http[^"]*' > login_url.txt
LOGIN_URL=$(cat login_url.txt)
SESSION_COOKIE=$(echo $LOGIN_URL | grep -o 'session_name=[^&]*&session_value=[^&]*' | sed 's/session_name=\([^&]*\)&session_value=\([^&]*\)/\1=\2/')

# Start the PHP server in background
php -S localhost:8888 -t web > /dev/null 2>&1 &
SERVER_PID=$!

# Wait for server to start
sleep 5

# Test accessing the module route
MODULE_URL="http://localhost:8888/admin/jsonapi-explorer"
HTTP_CODE=$(curl -s -o /dev/null -w "%{http_code}" -b "$SESSION_COOKIE" "$MODULE_URL")

# Kill the PHP server
kill $SERVER_PID

# Check the result
if [ "$HTTP_CODE" -eq 200 ]; then
  echo "SUCCESS: Module URL returned status code 200"
else
  echo "ERROR: Module URL returned status code $HTTP_CODE"
  exit 1
fi

# Clean up
echo "Cleaning up test directory..."
rm -rf "$TEST_DIR"

echo "All tests passed successfully!"
exit 0