#!/bin/bash

# Script to run PHPUnit tests for JSON:API Query Builder module in a proper Drupal environment

# Exit on error
set -e

# Default variables
MODULE_NAME="jsonapi_query_builder"
TEST_DIR="/tmp/drupal_test_phpunit_${MODULE_NAME}"
TESTSUITE="unit"  # Default to unit tests

# Process arguments
PHPUNIT_ARGS=""
while [[ $# -gt 0 ]]; do
  case $1 in
    --testsuite=*)
      TESTSUITE="${1#*=}"
      shift
      ;;
    --clean)
      CLEAN_ENV=true
      shift
      ;;
    --help)
      echo "Usage: $0 [options]"
      echo "Options:"
      echo "  --testsuite=SUITE   Run specific test suite (unit, functional, kernel, all)"
      echo "  --clean             Remove existing test environment and create a new one"
      echo "  --help              Show this help message"
      echo "  --                  Pass all remaining arguments to PHPUnit directly"
      exit 0
      ;;
    --)
      # All remaining args go to PHPUnit
      shift
      PHPUNIT_ARGS="$*"
      break
      ;;
    *)
      # Unknown option
      echo "Unknown option: $1"
      echo "Use --help for usage information"
      exit 1
      ;;
  esac
done

# Current module directory
MODULE_DIR=$(cd "$(dirname "$0")/.." && pwd)
echo "Module directory: $MODULE_DIR"

# Clean environment if requested
if [ "$CLEAN_ENV" = "true" ]; then
  echo "Cleaning up previous test environment..."
  rm -rf "$TEST_DIR"
fi

# Check if Drupal environment exists
if [ -d "$TEST_DIR" ] && [ -d "$TEST_DIR/web/core" ]; then
  echo "Using existing Drupal test environment at $TEST_DIR"
else
  echo "Creating new Drupal test environment at $TEST_DIR"
  mkdir -p "$TEST_DIR"
  
  echo "Installing Drupal with Composer..."
  cd "$TEST_DIR"
  
  # Create composer.json with stability settings first
  cat > composer.json << 'EOF'
{
  "name": "drupal/recommended-project",
  "description": "Project template for Drupal projects with a relocated document root",
  "type": "project",
  "license": "GPL-2.0-or-later",
  "homepage": "https://www.drupal.org/project/drupal",
  "support": {
      "docs": "https://www.drupal.org/docs/user_guide/en/index.html",
      "chat": "https://www.drupal.org/node/314178"
  },
  "repositories": [
      {
          "type": "composer",
          "url": "https://packages.drupal.org/8"
      }
  ],
  "require": {
      "composer/installers": "^2.0",
      "drupal/core-composer-scaffold": "^10.1",
      "drupal/core-project-message": "^10.1",
      "drupal/core-recommended": "^10.1"
  },
  "conflict": {
      "drupal/drupal": "*"
  },
  "minimum-stability": "dev",
  "prefer-stable": true,
  "config": {
      "allow-plugins": {
          "composer/installers": true,
          "drupal/core-composer-scaffold": true,
          "drupal/core-project-message": true,
          "phpstan/extension-installer": true,
          "dealerdirect/phpcodesniffer-composer-installer": true
      },
      "sort-packages": true
  },
  "extra": {
      "drupal-scaffold": {
          "locations": {
              "web-root": "web/"
          }
      },
      "installer-paths": {
          "web/core": ["type:drupal-core"],
          "web/libraries/{$name}": ["type:drupal-library"],
          "web/modules/contrib/{$name}": ["type:drupal-module"],
          "web/profiles/contrib/{$name}": ["type:drupal-profile"],
          "web/themes/contrib/{$name}": ["type:drupal-theme"],
          "drush/Commands/contrib/{$name}": ["type:drupal-drush"],
          "web/modules/custom/{$name}": ["type:drupal-custom-module"],
          "web/profiles/custom/{$name}": ["type:drupal-custom-profile"],
          "web/themes/custom/{$name}": ["type:drupal-custom-theme"]
      },
      "drupal-core-project-message": {
          "include-keys": ["homepage", "support"],
          "post-create-project-cmd-message": [
              "<bg=blue;fg=white>                                                         </>",
              "<bg=blue;fg=white>  Congratulations, you've installed the Drupal codebase  </>",
              "<bg=blue;fg=white>  from the drupal/recommended-project template!          </>",
              "<bg=blue;fg=white>                                                         </>",
              "",
              "<bg=yellow;fg=black>Next steps</>:",
              "  * Install the site: https://www.drupal.org/docs/installing-drupal",
              "  * Read the user guide: https://www.drupal.org/docs/user_guide/en/index.html",
              "  * Get support: https://www.drupal.org/support",
              "  * Get involved with the Drupal community:",
              "      https://www.drupal.org/getting-involved",
              "  * Remove the plugin that prints this message:",
              "      composer remove drupal/core-project-message"
          ]
      }
  }
}
EOF

  # Now install using the custom composer.json
  composer install --no-interaction
  
  echo "Installing test dependencies..."
  composer require --dev drupal/core-dev:^10.1
  # Note: This will install PHPUnit ^9.6 which is what Drupal 10 requires
  
  echo "Installing module dependencies..."
  # Add stability flags for jsonapi_schema using jq
  composer config extra.composer-stability-flags.drupal/jsonapi_schema 20
  
  # Install dependencies
  composer require drupal/jsonapi_schema:^1.0 drupal/gin:^3.0
  
  echo "Copying module code to Drupal modules directory..."
  mkdir -p "web/modules/contrib/$MODULE_NAME"
  
  # Copy module code excluding the vendor directory
  rsync -a --exclude=vendor --exclude=vendor/ "$MODULE_DIR/" "web/modules/contrib/$MODULE_NAME/"
  cp -r "$MODULE_DIR"/.git* "web/modules/contrib/$MODULE_NAME/" 2>/dev/null || true
fi

# Run the tests
cd "$TEST_DIR"

# Check if PHPUnit config exists
if [ ! -f "web/core/phpunit.xml.dist" ]; then
  echo "Error: PHPUnit configuration file not found at web/core/phpunit.xml.dist"
  ls -la web/core/
  exit 1
fi

# Run linting first
echo "Running linting checks:"
cd "web/modules/contrib/$MODULE_NAME"
composer lint || echo "Linting reported issues but continuing with tests"
cd "$TEST_DIR"

# Check if we have a custom PHPUnit configuration file
if [ -f "web/modules/contrib/$MODULE_NAME/phpunit.xml.dist" ]; then
  echo "Using custom PHPUnit configuration"
  CONFIG_PATH="web/modules/contrib/$MODULE_NAME/phpunit.xml.dist"
else
  echo "Using Drupal core PHPUnit configuration"
  CONFIG_PATH="web/core/phpunit.xml.dist"
fi

# Prepare test command
if [ "$TESTSUITE" = "all" ]; then
  TEST_CMD="vendor/bin/phpunit -c $(pwd)/$CONFIG_PATH web/modules/contrib/$MODULE_NAME/tests/"
else
  TEST_CMD="vendor/bin/phpunit -c $(pwd)/$CONFIG_PATH --testsuite=$TESTSUITE web/modules/contrib/$MODULE_NAME"
fi

# Add any additional PHPUnit arguments
if [ -n "$PHPUNIT_ARGS" ]; then
  TEST_CMD="$TEST_CMD $PHPUNIT_ARGS"
fi

echo "Running PHPUnit tests ($TESTSUITE):"
echo "$TEST_CMD"
eval "$TEST_CMD"

echo "Tests completed."