<?php

/**
 * @file
 * A script to test the JSON:API Query Builder module in a real Drupal environment.
 *
 * This script uses Drupal's testing framework to create a temporary Drupal site,
 * install the module, and verify its functionality.
 *
 * Usage: php scripts/test-module.php
 */

use Drupal\Core\DrupalKernel;
use Drupal\Core\Site\Settings;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Test\TestDatabase;
use Drupal\Core\Test\TestRunnerKernel;
use Drupal\Core\Installer\InstallerKernel;
use Drupal\Core\Database\Database;

// Force usage of the testing site directory.
$site_path = 'sites/simpletest';

// Change to the Drupal root directory if needed.
$drupal_root = getcwd();

// Check if we're in the module directory.
if (basename($drupal_root) === 'jsonapi_query_builder') {
  // We need to move up one level to the Drupal root.
  echo "Running from module directory. Please run this script from the Drupal root directory.\n";
  echo "E.g.: php modules/contrib/jsonapi_query_builder/scripts/test-module.php\n";
  exit(1);
}

// Prepare autoloading.
$autoloader = require_once $drupal_root . '/autoload.php';

// Bootstrap Drupal.
$kernel = new TestRunnerKernel('test', $autoloader, FALSE);
$kernel->boot();
$kernel->prepareLegacyRequest(Request::createFromGlobals());

// Create a test database prefix.
try {
  $test_db = new TestDatabase();
}
catch (\Exception $e) {
  echo "Failed to create test database: " . $e->getMessage() . "\n";
  exit(1);
}

// Define database connection info for the test site.
$connection_info = Database::getConnectionInfo('default');
$connection_info['default']['prefix'] = $test_db->getDatabasePrefix();

// Create settings for the test site.
$settings = [
  'hash_salt' => 'test',
  'container_yamls' => [],
  'database' => $connection_info,
  'settings' => [],
  'config_directories' => [],
  'file_public_path' => 'sites/default/files',
  'file_private_path' => 'sites/default/files/private',
];

// Save test settings.
$settings_file = $drupal_root . '/' . $site_path . '/settings.php';
if (!file_exists($site_path)) {
  mkdir($site_path, 0775, TRUE);
}
file_put_contents($settings_file, '<?php $settings = ' . var_export($settings, TRUE) . ';');

// Bootstrap the test site.
try {
  $request = Request::createFromGlobals();
  Settings::initialize($drupal_root, $site_path, $autoloader);
  $kernel = DrupalKernel::createFromRequest($request, $autoloader, 'test');
  $kernel->prepareLegacyRequest($request);
  $kernel->boot();
  $container = $kernel->getContainer();
  $container->get('module_handler');

  echo "Installing Drupal...\n";

  // Install Drupal.
  $install_kernel = InstallerKernel::create($autoloader);
  $install_kernel->prepareLegacyRequest($request);
  $install_kernel->boot();
  $install_container = $install_kernel->getContainer();

  // Set up profile and database connection.
  $install_container->get('database');
  $install_container->get('config.factory');

  // Install Drupal with a profile.
  $install_container->get('module_installer')->install(['system', 'user']);
  $install_container->get('config.installer')->installDefaultConfig('core', 'standard');

  // Install required modules.
  echo "Installing required modules...\n";
  $install_container->get('module_installer')->install(['jsonapi', 'jsonapi_schema']);

  // Install our module.
  echo "Installing JSON:API Query Builder module...\n";
  $module_path = $drupal_root . '/modules/contrib/jsonapi_query_builder';

  // Check if module exists.
  if (!file_exists($module_path)) {
    echo "Module directory not found at $module_path\n";
    echo "Please ensure the module is available at this path or adjust the path in this script.\n";
    exit(1);
  }

  $install_container->get('module_installer')->install(['jsonapi_query_builder']);

  // Verify module is enabled.
  if ($install_container->get('module_handler')->moduleExists('jsonapi_query_builder')) {
    echo "SUCCESS: JSON:API Query Builder module installed successfully!\n";

    // Test accessing the route.
    $route_provider = $install_container->get('router.route_provider');
    $route = $route_provider->getRouteByName('jsonapi_query_builder.app');

    if ($route) {
      echo "SUCCESS: Route 'jsonapi_query_builder.app' found.\n";
    }
    else {
      echo "ERROR: Route 'jsonapi_query_builder.app' not found.\n";
      exit(1);
    }

    // Check other aspects of module functionality here.
    echo "All tests passed. The module is working correctly.\n";
  }
  else {
    echo "ERROR: Failed to enable the JSON:API Query Builder module.\n";
    exit(1);
  }
}
catch (\Exception $e) {
  echo "An error occurred during testing: " . $e->getMessage() . "\n";
  echo "Stack trace: " . $e->getTraceAsString() . "\n";
  exit(1);
}
finally {
  // Clean up.
  if (file_exists($settings_file)) {
    unlink($settings_file);
  }

  // Clean up the test database.
  if (isset($test_db)) {
    $test_db->destroy();
  }
}
