#!/bin/bash

# Test installation script for the JSON:API Query Builder module
# This script will create a temporary Drupal site and install the module for testing

# Set variables
MODULE_DIR=$(pwd)
MODULE_NAME="jsonapi_query_builder"
TIMESTAMP=$(date +%s)
TEST_DIR="/tmp/drupal_test_${MODULE_NAME}_${TIMESTAMP}"
SERVER_PORT=8888

# Check if the required tools are installed
if ! command -v composer &> /dev/null; then
  echo "Error: Composer is not installed. Please install Composer and try again."
  exit 1
fi

# Create test directory
echo "Creating test directory at $TEST_DIR..."
mkdir -p "$TEST_DIR"
cd "$TEST_DIR"

# Create a new Drupal project
echo "Creating new Drupal project (this may take a few minutes)..."
# Check if directory isn't empty (could be /opt/drupal in CI environments)
if [ "$(ls -A .)" ]; then
  echo "Warning: Current directory is not empty. Cleaning it first..."
  # Keep .git directory if it exists (for CI environments that need it)
  find . -mindepth 1 -maxdepth 1 -not -name ".git" -exec rm -rf {} \; 2>/dev/null || true
fi
composer create-project drupal/recommended-project:^10.1 . --quiet
if [ $? -ne 0 ]; then
  echo "Failed to create Drupal project."
  exit 1
fi

# Install required dependencies
echo "Installing required modules and themes..."
composer require --quiet drupal/jsonapi_schema:^1.0@dev drupal/gin:^3.0@RC
if [ $? -ne 0 ]; then
  echo "Failed to install required dependencies."
  exit 1
fi

# Install Drush for site management
echo "Installing Drush..."
composer require --quiet drush/drush
if [ $? -ne 0 ]; then
  echo "Failed to install Drush."
  exit 1
fi

# Create module directory
echo "Setting up module directory..."
MODULE_TARGET_DIR="web/modules/contrib/$MODULE_NAME"
mkdir -p "$MODULE_TARGET_DIR"

# Copy module files
echo "Copying module files..."
cp -r "$MODULE_DIR"/* "$MODULE_TARGET_DIR/"
cp -r "$MODULE_DIR"/.git* "$MODULE_TARGET_DIR/" 2>/dev/null || true

# Remove unnecessary files from module directory
echo "Cleaning up module directory..."
rm -rf "$MODULE_TARGET_DIR/vendor" "$MODULE_TARGET_DIR/release" "$MODULE_TARGET_DIR/test-install.sh"

# Install Drupal with standard profile (includes article content type)
echo "Installing Drupal (this may take a minute)..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush site:install standard --yes --db-url=sqlite://sites/default/files/.ht.sqlite --account-name=admin --account-pass=admin --site-name="JSON:API Query Builder Test"
if [ $? -ne 0 ]; then
  echo "Failed to install Drupal."
  exit 1
fi

# Enable required modules
echo "Enabling JSON:API module..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y jsonapi
if [ $? -ne 0 ]; then
  echo "Failed to enable JSON:API module."
  exit 1
fi

echo "Enabling JSON:API Schema module..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y jsonapi_schema
if [ $? -ne 0 ]; then
  echo "Failed to enable JSON:API Schema module."
  exit 1
fi

echo "Enabling JSON:API Hypermedia module..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y jsonapi_hypermedia
if [ $? -ne 0 ]; then
  echo "Failed to enable JSON:API Hypermedia module."
  exit 1
fi

# Enable the Gin theme
echo "Installing and setting Gin as the admin theme..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush theme:enable gin
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush config:set system.theme admin gin -y
if [ $? -ne 0 ]; then
  echo "⚠️ WARNING: Failed to enable Gin theme, but continuing..."
fi

# Enable our module
echo "Enabling $MODULE_NAME module..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y $MODULE_NAME
if [ $? -ne 0 ]; then
  echo "Failed to enable $MODULE_NAME module."
  exit 1
else
  echo -e "\n✅ SUCCESS: $MODULE_NAME module installed successfully!"
fi

# Create test content would go here, but we're using standard profile's default content
echo "Using standard profile's default content..."

# Check if module routes exist
echo "Verifying module routes..."
ROUTE_CHECK=$(PHP_DEPRECATION_ERRORS=0 vendor/bin/drush route:info $MODULE_NAME.explorer 2>/dev/null)
if [[ $ROUTE_CHECK == *"$MODULE_NAME.explorer"* ]]; then
  echo -e "✅ SUCCESS: Route '$MODULE_NAME.explorer' found."
else
  echo "⚠️ WARNING: Route '$MODULE_NAME.explorer' not found."
fi

# Disable CSS and JS aggregation to prevent 404 errors
echo "Disabling CSS and JS aggregation..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush config:set system.performance css.preprocess 0 -y
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush config:set system.performance js.preprocess 0 -y

# Clear caches
echo "Clearing caches..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush cr

# Start a PHP server to test the module
echo -e "\n🚀 INSTALLATION COMPLETE!"
echo -e "\n📝 TEST SITE INFORMATION:"
echo "--------------------------------"
echo "📂 Site directory: $TEST_DIR"
echo "🔗 Admin URL: http://localhost:$SERVER_PORT/user/login"
echo "👤 Username: admin"
echo "🔑 Password: admin"
echo "🔍 JSON:API Query Builder URL: http://localhost:$SERVER_PORT/admin/jsonapi-query-builder"
echo "--------------------------------"
echo -e "\n🚀 Starting PHP server on port $SERVER_PORT..."
echo "Press Ctrl+C to stop the server when you're done testing."
echo -e "\n⏳ Server starting... please wait!"

# Start the PHP server
cd web
echo -e "\n⚠️  NOTE: To clean up the test directory later, run: rm -rf $TEST_DIR"
php -S localhost:$SERVER_PORT

# This part only runs after the server is stopped with Ctrl+C
echo -e "\n❓ Do you want to clean up the test directory? (y/n)"
read -r answer
if [[ $answer =~ ^[Yy]$ ]]; then
  echo "🧹 Attempting to clean up test directory..."
  # First change back to a safe directory
  cd /tmp
  # Use a more robust cleanup approach with sudo if needed
  if rm -rf "$TEST_DIR" 2>/dev/null; then
    echo "✅ Cleanup complete!"
  else
    echo "⚠️  Could not remove directory with regular permissions."
    echo "You may need to remove it manually later with:"
    echo "sudo rm -rf $TEST_DIR"
  fi
else
  echo "🛑 Test directory will be preserved at: $TEST_DIR"
fi