#!/bin/bash

# Script to test the module without jsonapi_hypermedia
# This is a simplified test that just enables the module without the hypermedia dependency

set -e  # Exit on error

# Variables
MODULE_DIR=$(pwd)
MODULE_NAME="jsonapi_query_builder"
TEST_DIR="/tmp/drupal_test_${MODULE_NAME}_nohypermedia"

# Create test directory
echo "Creating test directory at $TEST_DIR..."
mkdir -p "$TEST_DIR"
cd "$TEST_DIR"

# Create Drupal project
echo "Creating new Drupal project..."
composer create-project drupal/recommended-project:^10.1 . --quiet
if [ $? -ne 0 ]; then
  echo "Failed to create Drupal project."
  exit 1
fi

# Install required dependencies (without hypermedia)
echo "Installing required modules and themes..."
composer require --quiet drupal/jsonapi_schema:^1.0@dev drupal/gin:^3.0@RC
if [ $? -ne 0 ]; then
  echo "Failed to install required dependencies."
  exit 1
fi

# Install Drush
echo "Installing Drush..."
composer require --quiet drush/drush
if [ $? -ne 0 ]; then
  echo "Failed to install Drush."
  exit 1
fi

# Create module directory
echo "Setting up module directory..."
MODULE_TARGET_DIR="web/modules/contrib/$MODULE_NAME"
mkdir -p "$MODULE_TARGET_DIR"

# Copy module files
echo "Copying module files..."
cp -r "$MODULE_DIR"/* "$MODULE_TARGET_DIR/"
cp -r "$MODULE_DIR"/.git* "$MODULE_TARGET_DIR/" 2>/dev/null || true

# Remove unnecessary files from module directory
echo "Cleaning up module directory..."
rm -rf "$MODULE_TARGET_DIR/vendor" "$MODULE_TARGET_DIR/release" "$MODULE_TARGET_DIR/test-install.sh"

# Install Drupal
echo "Installing Drupal..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush site:install minimal --yes --db-url=sqlite://sites/default/files/.ht.sqlite --account-name=admin --account-pass=admin --site-name="JSON:API Query Builder Test"
if [ $? -ne 0 ]; then
  echo "Failed to install Drupal."
  exit 1
fi

# Enable required modules
echo "Enabling required modules..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y jsonapi jsonapi_schema
if [ $? -ne 0 ]; then
  echo "Failed to enable required modules."
  exit 1
fi

# Enable our module
echo "Enabling $MODULE_NAME module..."
PHP_DEPRECATION_ERRORS=0 vendor/bin/drush en -y $MODULE_NAME
if [ $? -ne 0 ]; then
  echo "Failed to enable $MODULE_NAME module."
  exit 1
else
  echo -e "\n✅ SUCCESS: $MODULE_NAME module installed successfully!"
fi

# Check if module routes exist
echo "Verifying module routes..."
ROUTE_CHECK=$(PHP_DEPRECATION_ERRORS=0 vendor/bin/drush route:info $MODULE_NAME.explorer 2>/dev/null)
if [[ $ROUTE_CHECK == *"$MODULE_NAME.explorer"* ]]; then
  echo -e "✅ SUCCESS: Route '$MODULE_NAME.explorer' found."
else
  echo "⚠️ WARNING: Route '$MODULE_NAME.explorer' not found."
  exit 1
fi

echo -e "\n🎉 TEST PASSED: The module works without jsonapi_hypermedia! 🎉"